@@ -44,6 +44,7 @@ from bson.timestamp import Timestamp
 from bson.tz_util import utc
 from bson.json_util import default, object_hook
 
+
 class TestJsonUtil(unittest.TestCase):
 
     def setUp(self):
@@ -72,9 +73,12 @@ class TestJsonUtil(unittest.TestCase):
         #   http://bugs.python.org/issue6105
         #
         # self.assertEqual("{\"ref\": {\"$ref\": \"foo\", \"$id\": 5}}",
-        #                  json.dumps({"ref": DBRef("foo", 5)}, default=default))
-        # self.assertEqual("{\"ref\": {\"$ref\": \"foo\", \"$id\": 5, \"$db\": \"bar\"}}",
-        #                  json.dumps({"ref": DBRef("foo", 5, "bar")}, default=default))
+        #                  json.dumps({"ref": DBRef("foo", 5)},
+        #                  default=default))
+        # self.assertEqual("{\"ref\": {\"$ref\": \"foo\",
+        #                              \"$id\": 5, \"$db\": \"bar\"}}",
+        #                  json.dumps({"ref": DBRef("foo", 5, "bar")},
+        #                  default=default))
 
     def test_datetime(self):
         # only millis, not micros
@@ -87,21 +91,22 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual(re.IGNORECASE, res.flags)
 
     def test_minkey(self):
-        self.round_trip({"m" : MinKey()})
+        self.round_trip({"m": MinKey()})
 
     def test_maxkey(self):
-        self.round_trip({"m" : MinKey()})
+        self.round_trip({"m": MinKey()})
 
     def test_timestamp(self):
-        res = json.dumps({"ts" : Timestamp(4, 13)}, default=default)
-        dct = json.loads(res);
+        res = json.dumps({"ts": Timestamp(4, 13)}, default=default)
+        dct = json.loads(res)
         self.assertEqual(dct['ts']['t'], 4)
         self.assertEqual(dct['ts']['i'], 13)
 
     def test_uuid(self):
         if not should_test_uuid:
             raise SkipTest()
-        self.round_trip({'uuid' : uuid.UUID('f47ac10b-58cc-4372-a567-0e02b2c3d479')})
+        self.round_trip(
+                {'uuid': uuid.UUID('f47ac10b-58cc-4372-a567-0e02b2c3d479')})
 
 if __name__ == "__main__":
     unittest.main()
